'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var icons = require('@chakra-ui/icons');
var react = require('@chakra-ui/react');
var React = require('react');
var debounce = require('@react-hook/debounce');
var Fuse = require('fuse.js');
var reactWindow = require('react-window');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var Fuse__default = /*#__PURE__*/_interopDefaultLegacy(Fuse);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var joinUrl = function () {
    var segments = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        segments[_i] = arguments[_i];
    }
    return segments.map(function (s) { return s.replace(/(^\/+|\/+$)/g, ""); }).join("/");
};

var BASE_URL = "https://fonts.kitefly.dev";
var cachedFonts;
var BaseUrl = React.createContext(BASE_URL);
var useFonts = function () {
    var _a = React.useState(cachedFonts), fonts = _a[0], setFonts = _a[1];
    var _b = React.useState(!cachedFonts), isLoading = _b[0], setIsLoading = _b[1];
    var baseUrl = React.useContext(BaseUrl);
    var getImageUrl = function (family) {
        return joinUrl(baseUrl, family.replace(/ /g, "") + "-400.png");
    };
    React.useEffect(function () {
        if (cachedFonts) {
            return;
        }
        fetch(joinUrl(baseUrl, "fonts.json"))
            .then(function (res) { return res.json(); })
            .then(function (fonts) {
            return fonts
                .filter(function (f) { return f.variants.includes("regular"); })
                .map(function (f) { return (__assign(__assign({}, f), { imageUrl: getImageUrl(f.family) })); });
        })
            .then(function (fonts) {
            setFonts(fonts);
            setIsLoading(false);
            cachedFonts = fonts;
            console.log("Loaded fonts:", fonts);
        })
            .catch(function (e) { return console.error("Error while fetching fonts:", e); });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [baseUrl]);
    return { isLoading: isLoading, fonts: fonts, getImageUrl: getImageUrl };
};

var useSearch = function (fonts, searchTerm) {
    var _a = React.useState(), results = _a[0], setResults = _a[1];
    var fuse = React.useRef(new Fuse__default['default'](fonts, {
        keys: ["family"],
    }));
    var search = debounce.useDebounceCallback(function () {
        if (searchTerm) {
            var results_1 = fuse.current.search(searchTerm);
            setResults(results_1.map(function (f) { return f.item; }));
        }
        else {
            setResults(fonts);
        }
    }, 100);
    React.useEffect(function () {
        search();
    }, [fonts, searchTerm, search]);
    return results;
};

var loadedImages = new Set();
var FontBrowserItem = function (_a) {
    var font = _a.font, fontRatio = _a.fontRatio, style = _a.style, onSelect = _a.onSelect, selectedFont = _a.selectedFont;
    var _b = React.useState(loadedImages.has(font.family)), isLoaded = _b[0], setIsLoaded = _b[1];
    var imageWidth = Math.floor(font.preview.width * fontRatio);
    var imageHeight = Math.floor(font.preview.height * fontRatio);
    var handleLoad = React.useCallback(function () {
        loadedImages.add(font.family);
        setIsLoaded(true);
    }, [font]);
    return (React__default['default'].createElement(react.Button, { isFullWidth: true, variant: "ghost", justifyContent: "start", style: style, onClick: function () { return onSelect(font); }, rounded: "none", bg: selectedFont === font.family ? "gray.200" : undefined },
        React__default['default'].createElement(react.Skeleton, { position: "absolute", top: 2, w: imageWidth + "px", h: imageHeight + "px", rounded: "md", isLoaded: isLoaded }),
        React__default['default'].createElement(react.Fade, { in: isLoaded },
            React__default['default'].createElement(react.Image, { w: imageWidth + "px", h: imageHeight + "px", src: font.imageUrl, alt: font.family, onLoad: handleLoad }))));
};

var FontBrowser = function (_a) {
    var _b = _a.fonts, fonts = _b === void 0 ? [] : _b, _c = _a.fontSize, fontSize = _c === void 0 ? 20 : _c, onSelect = _a.onSelect, selectedFont = _a.selectedFont;
    var listRef = React.useRef(null);
    var fontRatio = fontSize / 48;
    var ItemRenderer = React.useMemo(
    // eslint-disable-next-line react/display-name
    function () { return function (_a) {
        var data = _a.data, index = _a.index, style = _a.style;
        return (React__default['default'].createElement(FontBrowserItem, { key: data[index].family, style: style, font: data[index], fontRatio: fontRatio, onSelect: onSelect, selectedFont: selectedFont }));
    }; }, [onSelect, selectedFont, fontRatio]);
    React.useEffect(function () {
        var _a;
        (_a = listRef.current) === null || _a === void 0 ? void 0 : _a.scrollTo(0);
    }, [fonts, listRef]);
    return (React__default['default'].createElement(react.Box, null,
        React__default['default'].createElement(reactWindow.VariableSizeList, { ref: listRef, height: 240, itemCount: fonts.length, itemSize: function (index) {
                return Math.floor(fonts[index].preview.height * fontRatio) + 20;
            }, width: "100%", overscanCount: 10, itemData: fonts }, ItemRenderer)));
};
FontBrowser.displayName = "FontBrowser";

// eslint-disable-next-line react/display-name
var FontSearch = React__default['default'].forwardRef(function (_a, ref) {
    var value = _a.value, onChange = _a.onChange;
    return (React__default['default'].createElement(react.Box, { p: 2, borderBottomWidth: 1, borderBottomColor: "gray.100" },
        React__default['default'].createElement(react.InputGroup, { size: "sm" },
            React__default['default'].createElement(react.InputLeftElement, { pointerEvents: "none" },
                React__default['default'].createElement(icons.SearchIcon, { color: "gray.300" })),
            React__default['default'].createElement(react.Input, { type: "text", placeholder: "Search font...", rounded: "md", ref: ref, value: value, onChange: function (e) { return onChange(e.target.value); } }))));
});

// eslint-disable-next-line react/display-name
var Body = React__default['default'].forwardRef(function (_a, ref) {
    var selectedFont = _a.selectedFont, onSelect = _a.onSelect;
    var _b = React.useState(""), searchTerm = _b[0], setSearchTerm = _b[1];
    var fonts = useFonts().fonts;
    var searchResults = useSearch(fonts, searchTerm);
    var onClose = react.usePopoverContext().onClose;
    var handleSelect = React.useCallback(function (f) {
        onSelect(f);
        onClose();
    }, [onSelect, onClose]);
    return (React__default['default'].createElement(react.VStack, { align: "stretch", spacing: 0 },
        React__default['default'].createElement(FontSearch, { ref: ref, value: searchTerm, onChange: setSearchTerm }),
        React__default['default'].createElement(FontBrowser, { fonts: searchResults !== null && searchResults !== void 0 ? searchResults : [], onSelect: handleSelect, selectedFont: selectedFont })));
});
var FontPicker = function (_a) {
    var value = _a.value, onSelect = _a.onSelect, _b = _a.baseUrl, baseUrl = _b === void 0 ? BASE_URL : _b;
    var initialFocusRef = React__default['default'].useRef(null);
    var _c = React.useState(value), selectedFont = _c[0], setSelectedFont = _c[1];
    var getImageUrl = useFonts().getImageUrl;
    React.useEffect(function () { return setSelectedFont(value); }, [setSelectedFont, value]);
    var handleSelect = React.useCallback(function (font) {
        onSelect === null || onSelect === void 0 ? void 0 : onSelect(font.family);
        setSelectedFont(font.family);
    }, [setSelectedFont, onSelect]);
    return (React__default['default'].createElement(BaseUrl.Provider, { value: baseUrl },
        React__default['default'].createElement(react.Popover, { initialFocusRef: initialFocusRef, isLazy: true },
            React__default['default'].createElement(react.PopoverTrigger, null,
                React__default['default'].createElement(react.Button, { justifyContent: "space-between", rightIcon: React__default['default'].createElement(icons.ChevronDownIcon, null), variant: "solid", w: 60 }, selectedFont ? (React__default['default'].createElement(react.Image, { src: getImageUrl(selectedFont), h: 4 })) : ("Select a font"))),
            React__default['default'].createElement(react.PopoverContent, null,
                React__default['default'].createElement(react.PopoverArrow, null),
                React__default['default'].createElement(react.PopoverBody, { p: 0 },
                    React__default['default'].createElement(Body, { ref: initialFocusRef, onSelect: handleSelect, selectedFont: selectedFont }))))));
};

var buildFontUrl = function (family) {
    return "https://fonts.googleapis.com/css?family=" + family.replace(/ /g, "+");
};

exports.FontPicker = FontPicker;
exports.buildFontUrl = buildFontUrl;
